module System.Automation.Wget where
import Data.Maybe
import System.IO
-- import System.Info
import System.Cmd
import System.Process
import System.Exit
import Control.Exception
import Prelude hiding (catch)
import Network.URI hiding (uriToString)
import System.Directory
import System.Automation.Wget.Options


{-
-- By Network.URI
uriToString :: (String->String) -> URI -> ShowS
uriToString userinfomap URI { uriScheme=scheme
                            , uriAuthority=authority
                            , uriPath=path
                            , uriQuery=query
                            , uriFragment=fragment
                            } =
    (scheme++) . (uriAuthToString userinfomap authority)
               . (path++) . (query++) . (fragment++)
-}

uriAuthToString :: (String->String) -> (Maybe URIAuth) -> ShowS
uriAuthToString _           Nothing   = id          -- shows ""
uriAuthToString userinfomap
        (Just URIAuth { uriUserInfo = uinfo
                      , uriRegName  = regname
                      , uriPort     = port
                      } ) =
    ("//"++) . (if null uinfo then id else ((userinfomap uinfo)++))
             . (regname++)
             . (port++)

uriToString :: URI -> String
uriToString URI { uriScheme=scheme
                 , uriAuthority=authority
                 , uriPath=path
                 , uriQuery=query
                 , uriFragment=fragment
                 } =
    ((scheme++) . (uriAuthToString id authority)
               . (path++)) ""

deduceDomain :: URI -> String
deduceDomain URI { uriScheme=scheme
                 , uriAuthority=authority
                 , uriPath=path
                 , uriQuery=query
                 , uriFragment=fragment
                 } =
    ((scheme++) . (uriAuthToString id authority)) ""


emptyOptions :: Options
emptyOptions = Options [] Nothing Nothing Nothing Nothing Nothing Nothing

defaultOptions dir = (Options [SetDirectory (TopName dir)] Nothing
                     (Just CheckTimeStamp) Nothing (Just ForceToCreate) Nothing Nothing)

downloadFile :: String -> Options -> URI -> IO ()
downloadFile dir opt uri = wget (mergeOptions (defaultOptions dir) opt) (uriToString uri)


wget :: Options -> String -> IO ()
wget options args =
  catch (do
           command <- downloader
           err <- system  $ command ++ " " ++ show options ++ args
           -- ph <- runCommand  $ downloader ++ " -c -C off -N -x -P cache/" ++ scheme uri ++ "/" ++ dir ++ " " ++ opt ++ " " ++ uriToString uri
           -- err <- waitForProcess ph
           exitWith err)
        (\e -> if e == ExitException ExitSuccess then return () else putStrLn ("Exception!: " ++ show e))
  where
      downloader = do
          wgetPath <- findExecutable "wget"
          case wgetPath of
            Just path -> return path
            Nothing -> error "No wget found"
      -- downloader = if os == "darwin" then "/usr/local/bin/wget" else "wget"


downloadSiteOption = Options [Recursive DownloadRecursively, Recursive (Levels 10)] Nothing Nothing
                     Nothing Nothing Nothing (Just RelativeOnly)

-- Download all of the Site.  wget -r -L -l 10
-- --follow-ftp flag can download ftp's file too ... but same directory. So, I don't use this flag.
downLoadSite :: FilePath -> URI -> IO ()
downLoadSite dir uri = downloadFile (joinPath dir (init (uriScheme uri))) downloadSiteOption uri
  where
      joinPath :: FilePath -> FilePath -> FilePath
      joinPath a b
          | null a        = '/' : b
          | last a == '/' = a ++ b
          | otherwise     = a ++ "/" ++ b

