module System.Automation.Wget.Options where
-- import Data.Maybe
import Network.URI
-- import System.Automation.Wget (uriToString)

-- http://www.bookshelf.jp/texi/wget/wget-ja.html
-- http://www.geocities.jp/horiuchimasaru/wget.html
-- http://members.at.infoseek.co.jp/futora/wget153/


addToOptions :: Options -> OptionList -> Options
addToOptions (Options optlist stdinfo down proxy dir cookie follow) opt = (Options (optlist ++ opt) stdinfo down proxy dir cookie follow)
addToOptions (In _) opt = error "Infomation option can't use with other options."

mergeOptions :: Options -> Options -> Options
mergeOptions (Options optlist stdinfo down proxy dir cookie follow)
             (Options addOptlist addStdinfo addDown addProxy addDir addCookie addFollow)
             = Options (optlist ++ addOptlist)
                       (maybe stdinfo (Just . id) addStdinfo)
                       (maybe down (Just . id) addDown)
                       (maybe proxy (Just . id) addProxy)
                       (maybe dir (Just . id) addDir)
                       (maybe cookie (Just . id) addCookie)
                       (maybe follow (Just . id) addFollow)

data Options = In Info
             | Options OptionList (Maybe StdInfo) (Maybe DownloadStrategy)
                       (Maybe Proxy) (Maybe DirectoryHierarchy)
                       (Maybe SessionCookie) (Maybe Following)

instance Show Options where
    show (In info) = show info
    show (Options optlist stdinfo down proxy dir cookie follow)
         = show optlist ++ " "
         ++ showMaybeOption stdinfo ++ " " ++ showMaybeOption down
         ++ showMaybeOption proxy ++ " " ++ showMaybeOption dir ++ " "
         ++ showMaybeOption cookie ++ " " ++ showMaybeOption follow ++ " "

showMaybeOption opt = maybe "" (show) opt


type OptionList = [Option]
data Option = StartUp StartUpOptions | Log Logging | Inputs InputOptions | Download DownloadOptions | Authorize Login 
            | SetDirectory Directories | Http HTTP | Https HTTPS | Ftp FTP
            | Recursive RecursiveOptions | AcceptOrReject AcceptOrRejectOptions

instance Show Option where
    show (StartUp start) = show start
    show (Log log) = show log
    show (Inputs input) = show input
    show (Download download) = show download
    show (Authorize login) = show login
    show (SetDirectory directories) = show directories
    show (Http http) = show http
    show (Https https) = show https
    show (Ftp ftp) = show ftp
    show (Recursive recursive) = show recursive
    show (AcceptOrReject acceptReject) = show acceptReject
    showList (x:xs) s   = " " ++ (show x) ++ (showList xs s)
    showList [] s = s


data Info = Version | Help

instance Show Info where
    showsPrec _ x s = show x ++ s
    show Version = "--version" -- -V
    show Help = "--help" -- -h

data StartUpOptions =  BackGround | Execute String

instance Show StartUpOptions where
    showsPrec _ x s = show x ++ s
    show BackGround = "--background" -- -b
    show (Execute command) = "--execute " ++ command -- -e

data Logging = OutputFile FilePath | AppendOutput FilePath

instance Show Logging where
    showsPrec _ x s = show x ++ s
    show (OutputFile file) = "--output-file " ++ file -- -o
    show (AppendOutput file) = "--append-output " ++ file -- -a

data StdInfo = Debug | Quiet | Verbose | NonVerbose

instance Show StdInfo where
    showsPrec _ x s = show x ++ s
    show Debug = "--debug" -- -d
    show Quiet = "--quiet" -- -q
    show Verbose = "--verbose" -- -v
    show NonVerbose = "--non-verbose" --nv


data InputOptions = Input FilePath | ForceToHtml | Base URI

instance Show InputOptions where
    showsPrec _ x s = show x ++ s
    show (Input file) = "--input-file " ++ file -- -i
    show ForceToHtml = "--force-html" -- -F
    show (Base uri) = "--base " ++ show uri

data Try = Tries Int | UnlimitedRetry | RetryIfConnentionRefused

instance Show Try where
    showsPrec _ x s = show x ++ s
    show (Tries number) =
        if number > 0
          then "--tries " ++ (show number) -- -t
          else error "number of retries isn't valid."
    show UnlimitedRetry = "--tries 0"
    show RetryIfConnentionRefused  = "--retry-connrefused"

data Timeout = DefaultTimeout Int | NoTimeout | DNSTimeout Int | ConnectTimeout Int | ReadTimeout Int

instance Show Timeout where
    showsPrec _ x s = show x ++ s
    show (DefaultTimeout seconds) =
        if seconds > 0
          then "--timeout " ++ (show seconds) -- -T
          else error "second of timeout isn't valid."
    show NoTimeout = "--timeout 0"
    show (DNSTimeout seconds) = "--dns-timeout " ++ (show seconds)
    show (ConnectTimeout seconds) = "--connect-timeout " ++ (show seconds)
    show (ReadTimeout seconds) = "--read-timeout " ++ (show seconds)

data Wait = Time Int | RetryTime Int | Random

instance Show Wait where
    showsPrec _ x s = show x ++ s
    show (Time seconds) =
        if seconds >= 0
          then "--wait " ++ (show seconds) -- -w
          else error "second of wait time isn't valid."
    show (RetryTime seconds) =
        if seconds >= 0
          then "--waitretry " ++ (show seconds)
          else error "second of retry wait time isn't valid."
    show Random = "--random-wait"

data Proxy = Allow | No

instance Show Proxy where
    showsPrec _ x s = show x ++ s
    show Allow = "--proxy" -- -Y
    show No = "--no-proxy"

data Unit = Kilo | Mega

instance Show Unit where
    showsPrec _ x s = show x ++ s
    show Kilo = "k"
    show Mega = "m"

showUnit maybeUnit = case maybeUnit of
                  Nothing -> ""
                  Just unit -> show unit

data Size = FileSize Int (Maybe Unit) | BitRate Int (Maybe Unit)

instance Show Size where
    showsPrec _ x s = show x ++ s
    show (FileSize size unit) = "--quota " ++ show size ++ showUnit unit -- -Q
    show (BitRate size unit) = "--limit-rate " ++ show size ++ showUnit unit


data DNS = Address String | NoDNSCache

instance Show DNS where
    showsPrec _ x s = show x ++ s
    show (Address ip) = "--bind-address " ++ show ip -- -Q
    show NoDNSCache = "--no-dns-cache"


-- can't use together
data DownloadStrategy = NoClobber | Resume | CheckTimeStamp

instance Show DownloadStrategy where
    show NoClobber = "--no-clobber" -- -nc
    show Resume = "--continue" -- -c
    show CheckTimeStamp = "--timestamping" -- -N

data ProgressBar = Dot | Bar

instance Show ProgressBar where
    showsPrec _ x s = show x ++ s
    show Dot = "dot"
    show Bar = "bar"

data DownloadOptions = Re Try | Output FilePath | Progress ProgressBar | ServerRespose | CheckOnly
                     | TimeOut Timeout | WaitInterval Wait | Limited Size | BIND DNS | RestrictFileNameForOS String

instance Show DownloadOptions where
    showsPrec _ x s = show x ++ s
    show (Re try) = show try
    -- filpath use show for Windows path.
    show (Output filepath) = "--output-document " ++ show filepath -- -O
    show (Progress bar) = "--progress " ++ show bar
    show ServerRespose = "--server-response" -- -S
    show CheckOnly = "--spider"
    show (TimeOut timeout) = show timeout
    show (WaitInterval wait) = show wait
    show (Limited size) = show size
    show (BIND dns) = show dns
    show (RestrictFileNameForOS os) = "--restrict-file-names " ++ os

data Access = User String | Password String

instance Show Access where
    showsPrec _ x s = show x ++ s
    show (User user) = "user " ++ user
    show (Password pass) = "password " ++ pass

-- data Access a = Default a | HTTP a | FTP a | Proxy a
data Login = Default Access | HTTP Access | FTP Access | Proxy Access

-- instance (Show a) => Show (Login a) where
instance Show Login where
    showsPrec _ x s = show x ++ s
    show (Default a) = "--" ++ show a
    show (HTTP a) = "--http-" ++ show a
    show (FTP a) = "--ftp-" ++ show a
    show (Proxy a) = "--proxy-" ++ show a


data NoDirectory = Create | CreateHosts
data DirectoryHierarchy = DoNot NoDirectory | ForceToCreate

instance Show DirectoryHierarchy where
    showsPrec _ x s = show x ++ s
    show (DoNot Create) = "--no-directories" -- -nd
    show (DoNot CreateHosts) = "--no-host-directories" -- -nH
    show ForceToCreate = "--force-directories" -- -x


data Directories = Cut Int | TopName String

instance Show Directories where
    showsPrec _ x s = show x ++ s
    show (Cut numbers) = "--cut-dirs " ++ show numbers
    -- TopName use show for Windows path.
    show (TopName name) = "--directory-prefix " ++ show name -- -P

data UseCookie = Load String | Save String

instance Show UseCookie where
    showsPrec _ x s = show x ++ s
    show (Load cookie) = "--load-cookies " ++ cookie
    show (Save cookie) = "--save-cookies " ++ cookie

data SessionCookie = NoCookies | Use [UseCookie] | Keep

instance Show SessionCookie where
    showsPrec _ x s = show x ++ s
    show NoCookies = "--no-cookies"
    show (Use cookie) = show cookie
    show Keep = "--keep-session-cookies"

data PostData = Data String | File FilePath

instance Show PostData where
    showsPrec _ x s = show x ++ s
    show (Data string) = "--post-data " ++ string
    show (File file) = "--post-file " ++ file

data HTTP = NoCache | HTMLExtension | IgnoreContentLength | InsertStringAmongHeader String | SaveHeaders
          | Referer URI | UserAgent String | NoHTTPKeepAlive

instance Show HTTP where
    showsPrec _ x s = show x ++ s
    show NoCache = "--no-cache"
    show HTMLExtension = "--html-extension" -- -E
    show IgnoreContentLength = "--ignore-length"
    show (InsertStringAmongHeader string) = "--header " ++ string
    show SaveHeaders = "--save-headers"
    show (Referer uri) = "--referer" ++ show uri
    show (UserAgent ua) = "--user-agent " ++ ua -- -U
    show NoHTTPKeepAlive = "--no-http-keep-alive"

data ChoseSecureProtocol = SSLv2 | SSLv3 | TLSv1 deriving (Show)

data CertificateType = REM | DER deriving (Show)

data Certificate = Client FilePath CertificateType
                 | PrivateKey FilePath CertificateType
                 | CA FilePath FilePath

instance Show Certificate where
    showsPrec _ x s = show x ++ s
    show (Client file certype) = "--certificate " ++ file
                               ++ " --certificate-type " ++ show certype
    show (PrivateKey file certype) = "--private-key " ++ file
                                   ++ " --private-key-type " ++ show certype
    show (CA file dir) = "--ca-certificate " ++ file
                       ++ " --ca-directory " ++ dir


data HTTPS = SecureProtocol ChoseSecureProtocol | CertificateWay Certificate
           | NoCheck | Seed FilePath | EGD FilePath

instance Show HTTPS where
    showsPrec _ x s = show x ++ s
    show (SecureProtocol protocol) = "--secure-protocol " ++ show protocol
    show (CertificateWay cer) = show cer
    show (NoCheck) = "--no-check-certificate"
    show (Seed file) = "--ramdom-file " ++ file
    show (EGD file) = "--edg-file " ++ file

data FTP = NoRemoveListing | NoGlob | NoPassive | UseSymlink | Preserve

instance Show FTP where
    showsPrec _ x s = show x ++ s
    show NoRemoveListing = "--no-remove-listing"
    show NoGlob = "--no-glob"
    show NoPassive = "--no-passive-ftp"
    show UseSymlink = "--retr-symlinks"
    show Preserve = "--preserve-permissions"

data ConvertLinks = Links | LinksWithBackup

instance Show ConvertLinks where
    showsPrec _ x s = show x ++ s
    show Links = "--convert-links" -- -k
    show LinksWithBackup = show Links ++ " --backup-converted" -- -K

data RecursiveOptions = DownloadRecursively | Levels Int | AllLevels | Convert ConvertLinks
                      | DeleteAfter | Mirror | PageRequisites | StrictComments

instance Show RecursiveOptions where
    showsPrec _ x s = show x ++ s
    show DownloadRecursively = "--recursive" -- "-r"
    show (Levels number) =
        if number > 0
          then "--level " ++ (show number) -- "-l"
          else error "number of levels isn't valid."
    show AllLevels = "--level 0"
    show (Convert conv) = show conv
    show DeleteAfter = "--delete-after"
    show Mirror = "--mirror" -- -m
    show PageRequisites = "--page-requisites" -- -p
    show StrictComments = "--strict-comment"


data OtherHosts = FTPHosts | SpanHosts

instance Show OtherHosts where
    showsPrec _ x s = show x ++ s
    show FTPHosts = "--follow-ftp"
    show SpanHosts = "--span-hosts" -- -H

data Following = Follow [OtherHosts] | RelativeOnly

instance Show Following where
    showsPrec _ x s = show x ++ s
    show (Follow other) = show other
    show RelativeOnly = "--relative" -- "-L"

{-
data IE a = Include a | Exclude a
instance (Show a) => Show (IE a) where
    showsPrec _ x s = show x ++ s
    show (Include a) = "--include-" ++ show a
    show (Exclude a) = "--exclude-" ++ show a
-}

data AcceptOrRejectOptions = AcceptExtentions [String] | RejectExtentions [String]
                           | Domains [URI] | ExcludeDomains [URI]
                           | FollowTags [String] | IgnoreTags [String]
                           | IncludeDirectories [String] | ExcludeDirectories [String]
                           | NoParent

instance Show AcceptOrRejectOptions where
    showsPrec _ x s = show x ++ s
    show (AcceptExtentions strings) = "--accept " ++ showStrings strings -- -A
    show (RejectExtentions strings) = "--reject " ++ showStrings strings -- -R 
    show (Domains uris) = "--domains " ++  showURIs uris -- -D
    show (ExcludeDomains uris) = "--exclude-domains " ++  showURIs uris
    show (FollowTags strings) = "--follow-tags " ++ showStrings strings
    show (IgnoreTags strings) = "--ignore-tags " ++ showStrings strings
    show (IncludeDirectories strings) = "--include-directories " ++ unwords strings -- -I
    show (ExcludeDirectories strings) = "--exclude-directories " ++ unwords strings -- -X
    show NoParent = "--no-parent" -- -np

showStrings = tail .  concatMap (showString ",")
showURIs = showStrings . map show

{-
GNU Wget 1.10.2, a non-interactive network retriever.
Usage: wget [OPTION]... [URL]...

Mandatory arguments to long options are mandatory for short options too.

Startup:
  -V,  --version           display the version of Wget and exit.
  -h,  --help              print this help.
  -b,  --background        go to background after startup.
  -e,  --execute=COMMAND   execute a `.wgetrc'-style command.

Logging and input file:
  -o,  --output-file=FILE    log messages to FILE.
  -a,  --append-output=FILE  append messages to FILE.
  -d,  --debug               print lots of debugging information.
  -q,  --quiet               quiet (no output).
  -v,  --verbose             be verbose (this is the default).
  -nv, --no-verbose          turn off verboseness, without being quiet.
  -i,  --input-file=FILE     download URLs found in FILE.
  -F,  --force-html          treat input file as HTML.
  -B,  --base=URL            prepends URL to relative links in -F -i file.

Download:
  -t,  --tries=NUMBER            set number of retries to NUMBER (0 unlimits).
       --retry-connrefused       retry even if connection is refused.
  -O,  --output-document=FILE    write documents to FILE.
  -nc, --no-clobber              skip downloads that would download to
                                 existing files.
  -c,  --continue                resume getting a partially-downloaded file.
       --progress=TYPE           select progress gauge type.
  -N,  --timestamping            don't re-retrieve files unless newer than
                                 local.
  -S,  --server-response         print server response.
       --spider                  don't download anything.
  -T,  --timeout=SECONDS         set all timeout values to SECONDS.
       --dns-timeout=SECS        set the DNS lookup timeout to SECS.
       --connect-timeout=SECS    set the connect timeout to SECS.
       --read-timeout=SECS       set the read timeout to SECS.
  -w,  --wait=SECONDS            wait SECONDS between retrievals.
       --waitretry=SECONDS       wait 1..SECONDS between retries of a retrieval.

       --random-wait             wait from 0...2*WAIT secs between retrievals.
  -Y,  --proxy                   explicitly turn on proxy.
       --no-proxy                explicitly turn off proxy.
  -Q,  --quota=NUMBER            set retrieval quota to NUMBER.
       --bind-address=ADDRESS    bind to ADDRESS (hostname or IP) on local host.

       --limit-rate=RATE         limit download rate to RATE.
       --no-dns-cache            disable caching DNS lookups.
       --restrict-file-names=OS  restrict chars in file names to ones OS allows.

       --user=USER               set both ftp and http user to USER.
       --password=PASS           set both ftp and http password to PASS.

Directories:
  -nd, --no-directories           don't create directories.
  -x,  --force-directories        force creation of directories.
  -nH, --no-host-directories      don't create host directories.
       --protocol-directories     use protocol name in directories.
  -P,  --directory-prefix=PREFIX  save files to PREFIX/...
       --cut-dirs=NUMBER          ignore NUMBER remote directory components.

HTTP options:
       --http-user=USER        set http user to USER.
       --http-password=PASS    set http password to PASS.
       --no-cache              disallow server-cached data.
  -E,  --html-extension        save HTML documents with `.html' extension.
       --ignore-length         ignore `Content-Length' header field.
       --header=STRING         insert STRING among the headers.
       --proxy-user=USER       set USER as proxy username.
       --proxy-password=PASS   set PASS as proxy password.
       --referer=URL           include `Referer: URL' header in HTTP request.
       --save-headers          save the HTTP headers to file.
  -U,  --user-agent=AGENT      identify as AGENT instead of Wget/VERSION.
       --no-http-keep-alive    disable HTTP keep-alive (persistent connections).

       --no-cookies            don't use cookies.
       --load-cookies=FILE     load cookies from FILE before session.
       --save-cookies=FILE     save cookies to FILE after session.
       --keep-session-cookies  load and save session (non-permanent) cookies.
       --post-data=STRING      use the POST method; send STRING as the data.
       --post-file=FILE        use the POST method; send contents of FILE.

HTTPS (SSL/TLS) options:
       --secure-protocol=PR     choose secure protocol, one of auto, SSLv2,
                                SSLv3, and TLSv1.
       --no-check-certificate   don't validate the server's certificate.
       --certificate=FILE       client certificate file.
       --certificate-type=TYPE  client certificate type, PEM or DER.
       --private-key=FILE       private key file.
       --private-key-type=TYPE  private key type, PEM or DER.
       --ca-certificate=FILE    file with the bundle of CA's.
       --ca-directory=DIR       directory where hash list of CA's is stored.
       --random-file=FILE       file with random data for seeding the SSL PRNG.
       --egd-file=FILE          file naming the EGD socket with random data.

FTP options:
       --ftp-user=USER         set ftp user to USER.
       --ftp-password=PASS     set ftp password to PASS.
       --no-remove-listing     don't remove `.listing' files.
       --no-glob               turn off FTP file name globbing.
       --no-passive-ftp        disable the "passive" transfer mode.
       --retr-symlinks         when recursing, get linked-to files (not dir).
       --preserve-permissions  preserve remote file permissions.

Recursive download:
  -r,  --recursive          specify recursive download.
  -l,  --level=NUMBER       maximum recursion depth (inf or 0 for infinite).
       --delete-after       delete files locally after downloading them.
  -k,  --convert-links      make links in downloaded HTML point to local files.
  -K,  --backup-converted   before converting file X, back up as X.orig.
  -m,  --mirror             shortcut for -N -r -l inf --no-remove-listing.
  -p,  --page-requisites    get all images, etc. needed to display HTML page.
       --strict-comments    turn on strict (SGML) handling of HTML comments.

Recursive accept/reject:
  -A,  --accept=LIST               comma-separated list of accepted extensions.
  -R,  --reject=LIST               comma-separated list of rejected extensions.
  -D,  --domains=LIST              comma-separated list of accepted domains.
       --exclude-domains=LIST      comma-separated list of rejected domains.
       --follow-ftp                follow FTP links from HTML documents.
       --follow-tags=LIST          comma-separated list of followed HTML tags.
       --ignore-tags=LIST          comma-separated list of ignored HTML tags.
  -H,  --span-hosts                go to foreign hosts when recursive.
  -L,  --relative                  follow relative links only.
  -I,  --include-directories=LIST  list of allowed directories.
  -X,  --exclude-directories=LIST  list of excluded directories.
  -np, --no-parent                 don't ascend to the parent directory.

Mail bug reports and suggestions to <bug-wget@gnu.org>.
-}
