-----------------------------------------------------------
-- Short demo program in wxFruit.
--   Adapted from Short demo program sample in Hawiki.
--   http://haskell.org/hawiki/HaskellOpenGl
-----------------------------------------------------------

module Main where

import WXFruit
import AFRP
import Control.Monad
import Control.Arrow
import qualified Graphics.UI.WXCore as WXCore
import qualified Graphics.UI.WX as WX
import Data.Char
import Data.List ( transpose )
import Graphics.Rendering.OpenGL
import qualified Graphics.Rendering.OpenGL as GL

gameBox :: WX.Size
gameBox = WX.sz 350 250

angleControl :: GUI (Angles) (Angles)
angleControl = vBox $ proc anglesAux -> do
  rec _ <- text id -< (ttext ("Speed: " ++ (show vel))) 
      v <- slider Vert 1 10 (sselection 5) -< id
      let vel = fromIntegral v
          angles = addAngles anglesAux (Angles vel vel vel)
  returnA -< angles


rotateCube :: GUI () ()
rotateCube = hBox $ proc _ -> do
  rec anglesAux <- boxSF $ iPre (Angles 0 0 0) -< addAngles anglesDelta angles
      angles <- box angleControl -< anglesAux
      v <- boxSF $ iPre 0 -< 1 + v
      ev <- getEvent -< ()
      _ <- glpicture (glpsizeWithAttributes gameBox [WXCore.GL_RGBA, WXCore.GL_DOUBLEBUFFER]) -< glppicWithDialog (\f -> paintGL gameBox angles ev f)
  returnA -< ()

main :: IO ()
main = startGUI "Rotating Cube" rotateCube




convWG (WX.Size w h) = (GL.Size (convInt32  w) (convInt32  h))
convInt32 = fromInteger . toInteger

paintGL :: WX.Size -> Angles -> Event RawEvent -> WXCore.Frame () -> GLPicture
paintGL size v ev f glWindow dc myrect _ = do
   event (do
       -- This sets the current gl device context
       WXCore.glCanvasSetCurrent glWindow
       depthFunc $= Just Less
       reshape $ convWG $ WX.rectSize myrect
       display v glWindow
       return ())
     (keyboardMouse f) ev

data Angles = Angles Angle Angle Angle

anglesDelta :: Angles
anglesDelta = Angles 0.1 0.3 0.7

addAngles :: Angles -> Angles -> Angles
(Angles x1 y1 z1) `addAngles` (Angles x2 y2 z2) =
   Angles (x1 + x2) (y1 + y2) (z1 + z2)

updateInterval :: Int
updateInterval = 20

red, green, yellow, blue, purple, cyan :: Color4 GLfloat
red    = Color4 1 0 0 0
green  = Color4 0 1 0 0
yellow = Color4 1 1 0 0
blue   = Color4 0 0 1 0
purple = Color4 1 0 1 0
cyan   = Color4 0 1 1 0

incAnim :: Angles -> Angles
incAnim = proc angles -> do
   angle' <- addAngles anglesDelta -< angles
   returnA -< (angle')

reshape :: GL.Size -> IO ()
reshape size = do
   viewport $= (Position 0 0, size)
   matrixMode $= Projection
   loadIdentity
   ortho (-1) 1 (-1) 1 (-1) 1
   matrixMode $= Modelview 0
   loadIdentity

keyboardMouse :: (WX.Closeable w) => w -> WXFruit.RawEvent -> IO ()
keyboardMouse f (Key c _) =
   if WX.keyKey c == WX.KeyEscape
   || WX.keyKey c == WX.KeyChar (toUpper 'q')
     then WX.close f else return ()
keyboardMouse f _         = return ()

display :: Angles -> WXCore.GLCanvas a -> IO ()
display angles glWindow = do
   clear [ DepthBuffer, ColorBuffer ]
   loadIdentity
   let Angles angleX angleY angleZ = angles
   rotate angleX (Vector3 1 0 0)
   rotate angleY (Vector3 0 1 0)
   rotate angleZ (Vector3 0 0 1)
   zipWithM_ drawColoredQuad colors quads
   WXCore.glCanvasSwapBuffers glWindow
   -- GL.flush

drawColoredQuad :: Color4 GLfloat -> Quad GLfloat -> IO ()
drawColoredQuad c (Quad v1 v2 v3 v4) = do
   color c
   renderPrimitive Quads $ do
      vertex v1 ; vertex v2; vertex v3; vertex v4

colors :: [Color4 GLfloat]
colors = [ red, green, yellow, blue, purple, cyan ]

data Quad a = Quad (Vertex3 a) (Vertex3 a) (Vertex3 a) (Vertex3 a)

quads :: [Quad GLfloat]
quads = let x = 0.4; y = -x in [
   Quad (Vertex3 x x x) (Vertex3 y x x) (Vertex3 y y x) (Vertex3 x y x),
   Quad (Vertex3 x x y) (Vertex3 y x y) (Vertex3 y y y) (Vertex3 x y y),
   Quad (Vertex3 x x x) (Vertex3 y x x) (Vertex3 y x y) (Vertex3 x x y),
   Quad (Vertex3 x y x) (Vertex3 y y x) (Vertex3 y y y) (Vertex3 x y y),
   Quad (Vertex3 x x x) (Vertex3 x y x) (Vertex3 x y y) (Vertex3 x x y),
   Quad (Vertex3 y x x) (Vertex3 y y x) (Vertex3 y y y) (Vertex3 y x y)]

