--
-- |
-- Module      : Paint
-- Authors     : Sean Seefried (http://www.cse.unsw.edu.au/~sseefried)
-- Copyright   : (c) 2006
-- License     : BSD3
-- Created     : 12 Jan 2006
--
module Paint
where

-- library imports

import Data.IORef
import System.Random
import Utils



import Foreign
import System.Time


-- OpenGL
import Graphics.Rendering.OpenGL hiding (set,get)
import qualified Graphics.Rendering.OpenGL as OpenGL

-- wxHaskell
import Graphics.UI.WX hiding ( Size, marginWidth )
import Graphics.UI.WXCore hiding ( Size, marginWidth, Colour, Region)
import qualified Graphics.UI.WX as WX
import qualified Graphics.UI.WXCore as WXCore

-- Pan!
import Pan
{-
-- local imports
import GlobalData
import PanicConfAPI



--
-- Disable paint events
--
disablePaint :: IORef ViewerData  -> IO ()
disablePaint vwdRef = do
  vwd <- readIORef vwdRef
  let f = vwdEffectFrame vwd
  set f [ on paint := \dc rect -> return () ]

--
-- Enable paint events again
--
enablePaint :: IORef TopLevelData -> IORef ViewerData -> IO ()
enablePaint tldRef vwdRef = do
  vwd <- readIORef vwdRef
  let f = vwdEffectFrame vwd
  set f [ on paint :=
            \dc rect -> paintEffect tldRef vwdRef ]
-}
--
-- | Paints the effect to video memory
--
-- paintEffect :: IORef TopLevelData -> IORef ViewerData -> IO ()
-- paintEffect tldRef vwdRef = do
paintEffect size vtrans pixels displayFun t = do
  {-
  tld <- readIORef tldRef
  let pc = tldPanicConf tld
  vwd <- readIORef vwdRef
  let f = vwdEffectFrame vwd
  glCanvasSetCurrent (vwdGLCanvas vwd)
  -- Have paint events do nothing for the duration of this call.
  disablePaint vwdRef
  displayFun <- runUI (vwdDisplayFun vwd) (vwdUIRefs vwd)
  -}
  -- FIXME: Could optimise this.
  let sz@(w,h) = (WX.sizeW size, WX.sizeH size) -- vwdVSize vwd
      -- pixels = vwdPixels vwd
      -- maxAliasIterations' = maxAliasIterations pc
      k = 0 -- if vwdChanged vwd then 0 else vwdFramesSinceChange vwd
      -- vtrans = (0,0,1) -- vwdVTrans vwd
      -- frameText = vwdFrameStaticText vwd
  case displayFun of
    AnimDisplay animFun -> do
      -- t <- timeFromStart (vwdInitTime vwd)
      animFun pixels sz vtrans t
      -- setFramesPerSecond pc False
    ImageDisplay imageFun -> do
        {-
      if k < maxAliasIterations'
        then do
            -}
          xJitter <- randomRIO (0,1)
          yJitter <- randomRIO (0,1)
          imageFun pixels k xJitter yJitter sz vtrans
          {-
          setFramesPerSecond pc False
        else do
          setFramesPerSecond pc True
          -}
  return pixels
  {-
  drawPixels (Size (fromIntegral w) (fromIntegral h))
		 (PixelData RGBA UnsignedByte pixels)
  OpenGL.flush
  -- update frames since change and the "changed" flag
  updateVwd k pc
  -- Re-enable paint events
  enablePaint tldRef vwdRef
  where
    -- We can't set the frames per second every time paint gets called.
    -- Not only does this lead to too much updating for the eye to follow
    -- but it seems to swamp wxHaskell - perhaps too many events being
    -- processed?
    setFramesPerSecond pc doneFlag = do
      vwd <- readIORef vwdRef
      t <- timeFromStart (vwdLastSampleTime vwd)
      let frames = vwdFrames vwd + 1
      if t > timeSamplePeriod pc
        then do
          t' <- getClockTime
          let timeSamples = tail (vwdTimeSamples vwd) ++ [timeSample]
              timeSample = fromIntegral frames / t
	  writeIORef vwdRef
             (vwd { vwdLastSampleTime = t'
		  , vwdFrames = 0
		  , vwdTimeSamples = timeSamples
		  })
          let frameStaticText = vwdFrameStaticText vwd
          if doneFlag
            then do
              set frameStaticText [ text := "Done" ]
            else do
              let framesPerSecond =
                    (sum timeSamples) / (fromIntegral (samplesToStabilise pc))
                  framesText = showPlaces (placesToShow pc) framesPerSecond
	      set frameStaticText [ text := framesText ]
        else writeIORef vwdRef (vwd { vwdFrames = frames })
    updateVwd k pc = do
      vwd <- readIORef vwdRef
      let changed = vwdChanged vwd
          timeSamples = if changed
                          then take (samplesToStabilise pc) (repeat 0)
                          else vwdTimeSamples vwd
      writeIORef vwdRef (vwd { vwdFramesSinceChange = k + 1
                             , vwdChanged = False
                             , vwdTimeSamples = timeSamples }) -}