--
-- |
-- Module      : Panic
-- Author      : Sean Seefried (http://www.cse.unsw.edu.au/~sseefried)
-- Copyright   : (c) 2005
-- License     : BSD3
-- Created     :  31 March 2005
--
module Main
where

-- Standard Library imports
import Control.Concurrent.Chan
import Data.IORef
import Data.Char
import Monad
import GHC.Float
import System.IO
import System.Time
import System.Random
import System.Cmd
import System.Exit
import System.Directory
import Foreign hiding (rotate)

import Graphics.Rendering.OpenGL hiding (set,get)
import qualified Graphics.Rendering.OpenGL as OpenGL

import Data.Map as Map
import Control.Arrow

--
-- Package imports
--

-- wxhaskell
-- import Graphics.UI.WX hiding ( Size, marginWidth )
-- import Graphics.UI.WXCore hiding ( Size, marginWidth)
import qualified Graphics.UI.WX as WX
import qualified Graphics.UI.WXCore as WXCore

-- hs-plugins
-- import System.Plugins
-- import qualified System.Plugins as Plugins

-- WXFruit
import AFRP
import AFRPUtilities
import WXFruit hiding (startGUI, actuate)

-- Pan!
import Pan hiding (rotate, red, green, yellow, blue, animToDisplayFun, imageToDisplayFun)
import qualified Pan.Image as PanImage

-- local imports
-- import Consts
-- import PanicConfAPI
-- import GlobalData
import UIUtils
import UIUtilPrime
import Utils
import WXExts
import Paint
import Data.Array hiding (elems)

gameBox = WX.sz 250 150


-- pan :: GUI () ()
pan pixels = hBox $ proc _ -> do
  -- pixels <- returnA -< unsafePerformIO $ mallocBytes (250*(150+1)*bytesPerColour)
  let image = circle
  rec -- v <- boxSF $ iPre 0 -< 1 + v
      val <- boxSF $ time -< ()
      let v = realToFrac val
      ev <- getEvent -< ()
      vTrans <- boxSF $ iPre (0,0,1) -< (keyboardEvent 0.1 vTrans) ev
      panedPixels <- boxSF $ pictureEvent -< (pixels, vTrans, (image v), v, gameBox)
      _ <- glpicture (glpsizeWithAttributes gameBox [WXCore.GL_RGBA, WXCore.GL_DOUBLEBUFFER]) -< glppic $ paintGL gameBox panedPixels
      _ <- menu (mtextWithDialog "&SaveImage")
           -< mcommandWithDialog $
                (\v f -> saveImageHandler image (WX.sizeW gameBox, WX.sizeH gameBox) vTrans 1 v f) v
      {-
      fl <- menu (mtext "&SaveImage") -< (menabled True)
      _ <- command (cenabled False)
           -< ccommandEnabledWithDialog
                ((\v f -> saveImageHandler (imageToDisplayFun (image v)) image (WX.sizeW gameBox, WX.sizeH gameBox) 0 0 1 1 f) v) (if fl == NoEvent then False else True)
      -}
  returnA -< ()

pictureEvent :: SF (Ptr Int8, VTrans, ImageC, Frac, WXCore.Size) (IO (Ptr Int8))
pictureEvent = proc (pixels, vtrans, image, time, size) -> do
  returnA -< paintEffect size vtrans pixels (imageToDisplayFun image) time

{-
pictureEvent = proc (pixels, vtrans, image, time, size, rate) -> do
  -- glpicture <- pictureEvent' pixels time size -< rate
  -- glpicture <- returnA -< embedSynch pictureEvent' $ deltaEncode 10 (repeat (pixels, vtrans, image, time, size))
  -- glpicture' <- returnA -< pictureEvent' pixels time size
  -- glpicture <- returnA -< glpicture' rate
  returnA -< glpicture
-}


main = do
    pixels <- mallocBytes (250*150*bytesPerColour)
    startGUI "Panic" $ pan pixels

bitmapImage r = unsafePerformIO $ getImageCFromFile "..\\samples\\pls-small.png"
animImage t = unsafePerformIO $ animFromFile "c:/develop/cosmos/logo000.bmp" (Just 1) (Just 100) t

circle :: Frac -> ImageC
circle r = condR (circleReg r) blueI whiteI

-- circleReg :: Frac -> Image Bool
-- circleReg r (Pt x y) = distO (Pt x y) < r

circleReg :: Frac -> Region
circleReg r (Pt x y) = x*x + y*y < r*r


-- animToDisplayFun:: AnimC -> UI DisplayFun
animToDisplayFun anim = AnimDisplay $
    \ptr vsize vtrans t -> blitImageC (anim t) ptr vsize vtrans

-- imageToDisplayFun :: ImageC -> UI DisplayFun
imageToDisplayFun image = ImageDisplay $ antiAliasBlitImageC image


keyboardEvent zStep vwd ev = event vwd (keyboardEvent' zStep vwd) ev
keyboardEvent' zStep vwd (Key eventKey _) = effectKeyboardHandler zStep vwd eventKey
keyboardEvent' zStep vwd _ = vwd
--
-- Keyboard handler for the effect. Requires focus on the effect.
--
{-
effectKeyboardHandler :: IORef TopLevelData -> IORef ViewerData
                      -> EventKey -> IO ()
-}
effectKeyboardHandler tldRef vwdRef eventKey = -- do
    {-
  tld <- readIORef tldRef
  let panicConf = tldPanicConf tld
  -}
  let zStep = tldRef in -- zoomStep panicConf
  -- vwd <- readIORef vwdRef
  let vwd = vwdRef in
  case eventKey of
    WX.EventKey (WX.KeyChar '+') _ _  -> zoomIn  zStep vwd
    -- WX.EventKey (WX.KeyChar '=') _ _  -> zoomIn  zStep vwd
    WX.EventKey (WX.KeyChar '-')
         (WX.Modifiers _ True _ _) _  -> zoomIn  zStep vwd
    WX.EventKey (WX.KeyChar '-') _ _  -> zoomOut zStep vwd
    _                           -> vwd -- return ()
  where
   zoomIn  zStep (panX, panY, zoom) = (panX, panY, zoom*(1-zStep))
   zoomOut zStep (panX, panY, zoom) = (panX, panY, zoom*(1+zStep))
      {-
   zoomIn zStep vwd =
     let (panX, panY, zoom) = vwdVTrans vwd
     in writeIORef vwdRef (vwd { vwdVTrans = (panX, panY, zoom*(1-zStep))
                               , vwdChanged = True
                               })
   zoomOut zStep vwd =
     let (panX, panY, zoom) = vwdVTrans vwd
     in writeIORef vwdRef (vwd { vwdVTrans = (panX, panY, zoom*(1+zStep))
                               , vwdChanged = True
                               })
                               -}





convWG (WX.Size w h) = (OpenGL.Size (convInt32  w) (convInt32  h))
convInt32 = fromInteger . toInteger

-- This paint function gets the current glCanvas for knowing where to draw in.
-- It is possible to have multiple GL windows in your application.
-- paintGL :: WXCore.GLCanvas a -> WX.DC() -> WX.Rect -> [WX.Rect]-> IO ()
paintGL size pixels glWindow dc myrect _ = do

-- This sets the current gl device context
   WXCore.glCanvasSetCurrent glWindow
   -- myInit
   reshape $ convWG $ WX.rectSize myrect
   -- Or not reshape the size.
   -- reshape (GL.Size 320 210)
   -- viewport $= (Position 0 0, Size (fromIntegral 320) (fromIntegral 210))
   -- paintEffect size pixels (animToDisplayFun panImage) v
   pixel <- pixels
   drawPixels (convWG size)
		 (PixelData RGBA UnsignedByte pixel)
   OpenGL.flush
   WXCore.glCanvasSwapBuffers glWindow
   return ()

{-
paintGL' size v glWindow dc myrect _ = do

-- This sets the current gl device context
   WXCore.glCanvasSetCurrent glWindow
   -- myInit
   reshape $ convWG $ WX.rectSize myrect
   display v
   WXCore.glCanvasSwapBuffers glWindow
   return ()
-}

reshape size = do
   viewport $= (Position 0 0, size)
   matrixMode $= Projection
   loadIdentity
   ortho (-1) 1 (-1) 1 (-1) 1
   matrixMode $= Modelview 0
   loadIdentity


-- saveImageHandler :: IORef TopLevelData -> IORef ViewerData -> IO ()
-- saveImageHandler tldRef vwdRef = do
saveImageHandler img (wd,ht) vwdVTrans zoomFactor t = \f -> do
  let displayFun = animToDisplayFun img
  -- let displayFun = imageToDisplayFun (img t)
  {-
  tld <- readIORef tldRef
  vwd <- readIORef vwdRef
  let pc = tldPanicConf tld
  let f = vwdEffectFrame vwd
  displayFun <- runUI (vwdDisplayFun vwd) (vwdUIRefs vwd)
  -}
  case displayFun of
  {-
    AnimDisplay _ -> WX.infoDialog f "Animation can't be saved"
                     "Animations cannot be saved to image files"
                     -}
    AnimDisplay _ -> do
      saveConsecutiveNumberFiles f
        (\filePath imageFileType t -> saveImage' img (wd,ht) vwdVTrans zoomFactor filePath imageFileType t) t
    ImageDisplay image -> do
      mbResult <- fileSaveDialogImage f True True "Enter name"
      case mbResult of
        Just (filePath, imageFileType) -> do
            saveImage image (wd,ht) vwdVTrans zoomFactor filePath imageFileType
        Nothing -> return ()
  where
      saveImage' img (wd,ht) vwdVTrans zoomFactor filePath imageFileType t
        = saveImage (antiAliasBlitImageC (img t)) (wd,ht) vwdVTrans zoomFactor filePath imageFileType
      saveImage image (wd,ht) vwdVTrans zoomFactor filePath imageFileType = do
           let (panX, panY, zoom) = vwdVTrans -- vwd
        {-
               (w,h) = vwdVSize vwd
               (wd,ht) = vwdSaveSize vwd
               -- Zoom factor is based on y axis
               zoomFactor = fromIntegral h / fromIntegral ht
               -}
#ifndef mingw32_HOST_OS
           progress <- WXCore.progressDialogCreate "Progress" "Generating image..."
                         20 f 0 -- (maxAliasIterations pc) f 0
           WXCore.progressDialogResume progress
#else
#endif
           wxImage <- imageDisplayFunToWXImage
                         image 20 -- (maxAliasIterations pc)
                         (wd,ht)
                         (panX, panY, zoom * zoomFactor)
#ifndef mingw32_HOST_OS
                         (\k -> do { WXCore.progressDialogUpdate progress k
                                ; return ()})
#else
                         (\k -> return ())
#endif
           -- WXCore.imageSaveFile wxImage filePath (WXCore.imageTypeFromFileName filePath)
           WXCore.imageSaveFile wxImage filePath imageFileType
#ifndef mingw32_HOST_OS
           WXCore.windowDestroy progress
#else
#endif
           return ()
--         Nothing -> return ()

{-
saveImage :: Int -> Int -> FilePath -> ImageC -> IO ImageC
saveImage width height fileName img = do
    image <- WXCore.imageCreateFromPixels (WXCore.sz width height) $ pixelizeImage  width height
    WXCore.imageSaveFile image fileName (WXCore.imageTypeFromFileName fileName)
    WXCore.imageDelete image
    return img
    where
        pixelizeImage  width height =
            map (colourToColor . (\ (x, y) -> (img (Pt x y))) . (\ (x, y) -> (fromIntegral x, fromIntegral y)))
            $ range ((0,0), (width, height))
        colourToColor (Col r g b _) = WXCore.colorRGB (colourToColor' r) (colourToColor' g) (colourToColor' b)
        colourToColor' x = (fixedColor . fromEnum . (*256)) x
        fixedColor x = if x == 0 then 0 else x - 1
-}

-- Start a WXGUI running in a top-level frame.
startGUI :: String -> GUI () () -> IO ()
startGUI title (GUI g) = WX.start $ do
  -- Set up frame window and internal state.
  ch <- newChan
  f <- WX.frame [WX.text WX.:= title]
  defaultWindowsRawEvent f ch
  epoch <- getCurrentTime  
  gsr <- newIORef (f,Map.empty, epoch,CEmpty)
  
  -- Initialize Yampa.
  rh <- reactInit (initSense f) (actuate ch gsr) g
  
  -- Set up heartbeat timer to make periodic calls to react.
  -- This is necessary for animation.  Note that this is
  -- not the only place react is called from: button
  -- presses and other user actions may generate calls of
  -- their own.
  timer <- WX.timer f [WX.interval WX.:= 120, WX.on WX.command WX.:= respond ch gsr rh noEvent]
  
  return ()


-- Process an output sample (i.e. a widget request).
actuate :: Chan RawEvent -> GUIRef -> RHandle -> Bool -> (Event WidgetReq,()) -> IO Bool
actuate ch gsr rh _ (wre,_) = 
  do -- Handle requests, if any.
     (f,mu,t,prevc) <- readIORef gsr
     (resp,c,cch) <- handleWidgetReq ch gsr rh f [] prevc wre
     (f,mu,t,prevc) <- readIORef gsr

     {-
     -- create file menu
     file   <- WX.menuPane      [WX.text WX.:= "&File"]
     -- save   <- WX.menuItem file [WX.text WX.:= "&Save Image"]
     quit   <- WX.menuQuit file [WX.help WX.:= "Quit the demo"]
     -}
     -- create Help menu
     hlp    <- WX.menuHelp      []
     about  <- WX.menuAbout hlp [WX.help WX.:= "About ImageViewer"]
     
     -- Reset layout if contents changed.
     if cch
       then do WX.set f $ [WX.layout WX.:= WX.margin 5 (contents2layout c)]
                        ++ if Map.null mu
                             then []
                             else [WX.menubar WX.:= (elems mu ++ [hlp])
                                  -- ,WX.on (WX.menu quit)  WX.:= WX.close f
                                  ,WX.on (WX.menu about)  WX.:= WX.infoDialog f "Pan on Yampa"
                                         "This is a variation of Pan(ic) what implemeted by Yampa."]
               WX.windowReLayoutMinimal f
       else return ()
     writeIORef gsr (f,mu,t,c)
     
     -- Turn around and respond to the widgets, if necessary.
     -- Note that this causes a reentrant call to react.
     case resp of
       NoEvent -> return ()
       _ -> respond ch gsr rh resp
     
     return False
