{-# OPTIONS -fallow-undecidable-instances #-}
{-# OPTIONS -fallow-overlapping-instances #-}
--
-- |
-- Module      : UIUtilPrime
-- Author      : shelarcy (http://page.freett.com/shelarcy)
-- Copyright   : (c) 2005-2006
-- License     : BSD3
-- Created     : 1 Jul 2005
--
module UIUtilPrime
where

-- library imports
import Data.Char
import Data.List
import Data.Maybe
import System.Directory
import GHC.Float

-- wxHaskell
import Graphics.UI.WX hiding ( Size, marginWidth, value, dialog)
import Graphics.UI.WXCore hiding ( Size, marginWidth, Colour, Region, Time, FileName)
import qualified Graphics.UI.WX as WX
import qualified Graphics.UI.WXCore as WXCore

-- AutoForms
import qualified AutoForm as AF
import WxFormAll
import MySYB


-- Pan!
import Pan

-- local
import Utils
import UIUtils
import WXExts
import Paint

data Consecutive = Consecutive { directory :: AFDirectoryPath, fileName :: FileName
                               , currentTime :: Pan.Time, consecutiveNumbers :: Numbers }     deriving (Show, Read, Eq)
newtype FileName = FileName String         deriving (Show, Read, Eq)
data Numbers = Numbers { min   :: Int
                       , max :: Int
                       } deriving (Show, Read, Eq)
$(derive [''Consecutive, ''FileName, ''Numbers])

instance AF.TypePresentation Numbers tp1 tp2 tp3 where
    mkCom x = ( AF.label "Numbers" .
                AF.limit (\(Numbers min max) ->
                              return (min <= max)
                          )
                          ("min <= max") -- Error message
               ) (AF.afMkCom x)


saveConsecutiveNumberFiles f fun time' = do
    curr <- getCurrentDirectory
    mbResult <- AF.modalForm (toParent f) (AF.mkCom (Consecutive (AFDirectoryPath curr) (FileName "untitle.png") time' (Numbers 0 0)))
    case mbResult of
       Just (Consecutive (AFDirectoryPath dir) (FileName file) time (Numbers min max)) -> do
           let numbers = [min .. max]
               current num = (float2Int time) + (fromJust $ elemIndex num numbers)
               file' = takeSuffix file
               fileTypeExt = if null file' then ".png" else "." ++ file'
               imageFileType = imageTypeFromFileName fileTypeExt
           mapM_ (\num -> fun (dir ++ "/" ++ (dropSuffix file) ++ show num ++ fileTypeExt) imageFileType $ int2Float $ current num) numbers
       Nothing -> return ()

takeSuffix :: FilePath -> FilePath
takeSuffix f = if elem '.' f
                 then reverse . takeWhile (/= '.') $ reverse f
                 else ""


splitConsecutiveNumber :: String -> (String, String, String)
splitConsecutiveNumber str =
  let (ext, dropSuffStr) = break (== '.') $ reverse str
      base = reverse $ dropWhile (isDigit) $ drop 1 dropSuffStr
      num = reverse $ takeWhile (isDigit) $ drop 1 dropSuffStr
  in (base, num, reverse ext)


consecutiveNumberFiles :: FilePath -> (Maybe Int) -> (Maybe Int) ->IO [FilePath]
consecutiveNumberFiles filePath  withStart withLast = do
    let (base, num, ext) = splitConsecutiveNumber filePath
    if null num
      then return [filePath]
      else do
        files <- getFiles $ dirname base
        let firstAndThird (one, two, three) = (one, three)
            second (one, two, three) = two
            number = read num :: Int
        case withStart of
            Nothing ->
              case withLast of
                  Nothing ->
                    return $ filter ((\x -> ((>= number) $ read $ second x) && ((>= (base, ext)) $ firstAndThird x)) . splitConsecutiveNumber) files
                  Just last ->
                    return  $ filter ((\x -> ((<= last) $ read $ second x) && ((>= number) $ read $ second x) && ((== (base, ext)) $ firstAndThird x)) . splitConsecutiveNumber) files
            Just start ->
              case withLast of
                   Nothing ->
                     return $ filter ((\x -> ((>= start) $ read $ second x) && ((>= (base, ext)) $ firstAndThird x)) . splitConsecutiveNumber) files
                   Just last ->
                    return  $ filter ((\x -> ((<= last) $ read $ second x) && ((>= start) $ read $ second x) && ((== (base, ext)) $ firstAndThird x)) . splitConsecutiveNumber) files


-- Return the files of a certain directory: the full path.
-- this function come from wxHaskell sample
getFiles :: FilePath -> IO [FilePath]
getFiles fpath
  = do contents  <- getDirectoryContents fpath `catch` \err -> return []
       let names = filter (\file -> head file /= '.') contents
           paths = map (\file -> fpath ++ file) names
       isfile    <- mapM (\file -> doesFileExist file `catch` \err -> return False) paths
       let files  = [path | (isfile, path) <- zip isfile paths, isfile]
       return files


animFromFile :: FilePath -> Maybe Int -> Maybe Int -> Time -> IO ImageC
animFromFile filePath withStart withLast t = do
    filePathes <- consecutiveNumberFiles filePath withStart withLast
    let filesLength = length filePathes
        fileIndex = (float2Int t)
        currentFile = if fileIndex < filesLength
                        then filePathes!!fileIndex
                        else last filePathes
    bmpArray <- loadBitmap currentFile
    return (reconstruct bmpArray)
