-----------------------------------------------------------
-- PaddleBall in wxFruit.
--   Adapted from paddleball sample in the original Fruit.
--   Adds a slider controlling the velocity of the ball
--   and a restart button.
-----------------------------------------------------------

module Main where

import WXFruit
import AFRP
import AFRPUtilities
import Control.Arrow
import qualified Graphics.UI.WX as WX

-- Elements of the game.

bg :: Picture
bg = withColor WX.white fill

walls :: Picture
walls = let upper = picFill (rect (WX.rect (WX.point 10 10) (WX.sz 200 10)))
            left = picFill (rect (WX.rect (WX.point 10 10) (WX.sz 10 200)))
            right = picFill (rect (WX.rect (WX.point 200 10) (WX.sz 10 200)))
        in withColor WX.red (upper `picOver` left `picOver` right)

paddle :: Int -> WX.Rect
paddle xpos = WX.rect (WX.Point (xpos - 25) 200) (WX.sz 50 10)

gameBox = WX.sz 215 215

-- The game screen: takes signals for the velocity of the
-- ball and the current mouse position; outputs a picture
-- of the game and a game-over event.
pball :: SF (Double,WX.Point) (Picture,Event ())
pball = proc (vel,mpos) -> do
  rec xi <- integral -< xvel
      yi <- integral -< yvel
      let xpos = 30 + xi
      let ypos = 30 + yi
      let ballS = WX.rect (WX.point (round (xpos - 12.5)) (round (ypos - 12.5)))
                          (WX.sz 25 25)
      let ballPicS = withColor WX.yellow $ picFill $ ellipse ballS
      xbounce <- edge -< ((xpos > 175) || (xpos < 45))
      ybounce <- edge -< ((ypos < 45) || hitPaddle)
      let hitPaddle = WX.rectOverlaps ballS paddleS
      xdir <- accumHold 1 -< xbounce `tag` negate
      ydir <- accumHold 1 -< ybounce `tag` negate
      let xvel = xdir * vel
      let yvel = ydir * vel
      let paddleS = paddle (WX.pointX mpos)
      let paddlePicS = withColor WX.green $ picFill $ rect paddleS
      gameOver <- edge -< ypos > 250
      let gamePic = walls `picOver` paddlePicS `picOver` ballPicS `picOver` bg
  returnA -< (gamePic,gameOver)

-- The game-over screen.
death :: SF a (Picture,Event b)
death = proc _ -> do
  returnA -< (deathPic,noEvent)
    where deathPic = write "Game Over!" (WX.Point 70 100)

-- The game GUI: wrapper around the game-on and
-- and game-over screens.
pbgame :: GUI (Double,Event ()) (Event ())
pbgame = hBox $ proc (vel,restart) -> do
  rec let next = (gameOver `tag` death) `merge` (restart `tag` pball)
      mpos <- mouse -< ()
      (gamePic,gameOver) <- boxSF (drSwitch pball) -< ((vel,mpos),next)
  _ <- picture (psize gameBox) -< ppic gamePic
  returnA -< gameOver

-- The game-control GUI: widgets for controlling
-- the speed of the ball and restarting the game.
pbcontrol :: GUI (Event ()) (Double,Event ())
pbcontrol = vBox $ proc gameOver -> do
  rec _ <- text id -< (ttext ("Speed: " ++ (show vel))) 
      v <- slider Vert 5 15 (sselection 10) -< id
      let vel = v * 10
      stopped <- boxSF (hold False)
        -< (playAgain `tag` False) `merge` (gameOver `tag` True)
      playAgain <- menu (mtext "&PlayAgain") -< menabled stopped
  returnA -< (fromIntegral vel,playAgain)

-- The top-level GUI: puts it all together.
paddleBall :: GUI () ()
paddleBall = hBox $ proc _ -> do
  rec gameOver <- box pbgame -< (vel,playAgain)
      (vel,playAgain) <- box pbcontrol -< gameOver
  returnA -< ()

main :: IO ()
main = startGUI "PaddleBall" paddleBall
